# -*- coding:latin-1 -*-
"""
Copyright 2009 Sbastien L. W. Baelde (contact : clic4@clic4.org)

    This file is part of ImASCII.

    ImASCII is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    ImASCII is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ImASCII.  If not, see <http://www.gnu.org/licenses/>.
"""

from PIL import Image, ImageDraw, ImageFont
from PyRTF import Document, Section, Renderer, Font, TEXT, LINE, Paragraph, Colour
import math, os

def OpenFile( name ) :
	return file( '%s.rtf' % name, 'w' )

class Vecteur2D:
	def __init__ (self, x, y):
		self.x = x
		self.y = y
		
class Couleur:
	""" Classe grant une couleur. """
	def __init__(self, r, v, b):
		"""
		Constructeur.
		
		@param r: Couleur rouge (0-255)
		@param v: Couleur vert (0-255)
		@param b: Couleur bleu (0-255)
		"""
		self.r = int(r)
		self.v = int(v)
		self.b = int(b)

	def __str__(self):
		return "r: "+ str(self.r) + " v: " + str(self.v) + " b: " + str(self.b)

	def __eq__ (self, coul):
		
		#if coul.r == self.r and coul.v == self.v and coul.b == self.b:
		# Tolrance sur l'galit de couleur. Diminue le nombre de couleurs.
		if coul.r <= self.r + 35 and coul.r >= self.r - 35: 
			if coul.v <= self.v + 35 and coul.v >= self.v - 35:
				if coul.b <= self.b + 35 and coul.b >= self.b - 35:
					return True
		else:
			return False

class ImageASCII:
	"""
	Classe pour effectuer la conversion d'une image en texte.
	"""
	def __init__(self, parent):
		"""
		Construction d'une table de rfrence pour les caractres comprenant la valeur moyenne pour chaque lettre.
		Un espace est quivalent  un poids de 1.0 tandis qu'un caractre compltement noir aurait la valeur de 0.0
		
		@param parent: Fentre parent.
		"""
		self.parent = parent
		self.listeCaracteresA = ['.', ' ', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',  'i', 'j', 'k', 'l', 'm', 'n', 'o', 
						'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F', 
						'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 
						'X', 'Y', 'Z', ',', ';', ':', '?', '!', '+', '-', '*', '&', '/', '=', '(', ')', '<', 
						'>', '_', '[', ']', '{', '}', '', '^', '|', '', '', '', '#', '@', '%', '0', 
						'1', '2', '3', '4', '5', '6', '7', '8', '9', '\'', '\"', '~', '', '', '', '', '']
		self.listeCaracteresB = [' ', '.', 'o',  ',', ';', ':', '?', '%', '!', '+', '-', '*', '(', ')', '<', '>', '_', '[', ']', '{', '}', '^', '' ]
		self.listeCaracteresPerso = []
		# Liste courante de caractres
		self.listeCaracteres = self.listeCaracteresA
		
		# Dictionnaire des caractres et de leur poids respectif
		self.poidsNBCarac = {}
		# Police utilise pour la conversion (monospace)
		if os.name == 'nt':
			self.police = 'Lucida Console'
		else:
			self.police = 'Bitstream Vera Sans Mono'
		# Nom de l'image  convertir. Par dfaut, logo de ImageASCII
		self.nomImage = "img/guttenberg.png"
		# Nombre de caractres utiliss pour reprsenter l'image (tableau 2D nbrCarac x nbrCarac)
		self.nbrCarac = 120.0
		# Tolrance sur le poids. Dtermine la marge d'erreur.
		self.tolerancePoids = 0.05
		# Prcision de la tolrance de conversion.
		self.precisionTolerance = 100.0
		# Caractre par dfaut, quand aucune correspondance n'est trouve.
		self.caractereDefaut = '-'
		# Calculer le poids pour la liste courante de caractres.
		self.calculerPoids()

	def calculerPoids (self, tailleCarac=20):
		"""
		Calculer le poids de chaque caractre, selon la liste de rfrence.
		
		@keyword tailleCarac: Taille des caractres en pixels pour calculer le poids.
		"""
		self.poidsNBCarac = {}
		for c in self.listeCaracteres:
			poids = 0.000001
			# Dessiner le caractre 'c' sur une image de 20x20 pixels
			font = ImageFont.truetype(self.parent.listeFonts[self.police], tailleCarac)
			img = Image.new('RGB', (20, 20), (255, 255, 255)) 
			carac = ImageDraw.Draw(img)
			carac.text((0, 0), c, font=font, fill=(0, 0, 0) )
			# La mthode load renvoit un tableau en 2 dimensions comprennant les valeurs RVB de chaque pixel.
			res = img.load()

			# Parcourir les pixels et ajouter les valeurs
			for y in range(20):
				for x in range(20):
					pix = res[x, y]
					# Moyenne des valeurs sur les trois canaux.
					p = ( pix[0]/255.0  + pix[1]/255.0 + pix[2]/255.0 ) / 3.0
					# Additionner la valeur moyenne du pixel au poids total du caractre
					poids += p
			# Poids moyen pour le caractre.
			poidsMoyen = float(poids) / float( img.size[0] * img.size[1] )
			# Mmoriser le rsultat dans la liste des poids selon le caractre.
			self.poidsNBCarac[c] = poidsMoyen

		# Normaliser les poidsMoyen entre 0 et 1
		# Il n'y a pas de caractre entirement noir, le poids n'est donc pas une valeur comprise entre 0 et 1.
		# On corrige les poids en fonction du plus petit et du plus grand poids qui servent respectivement 
		# de 0 et de 1.
		pMin = 1.0
		pMax = 0.00001
		# Chercher le plus petit et le plus grand poids.
		for c in self.poidsNBCarac:
			if self.poidsNBCarac[c] < pMin:
				pMin = self.poidsNBCarac[c]
			if self.poidsNBCarac[c] > pMax:
				pMax = self.poidsNBCarac[c]
		# Modifier le poids de chaque caractre en le normalisant entre le poids minimum et maximum.
		for c in self.poidsNBCarac:
			self.poidsNBCarac[c] = ( self.poidsNBCarac[c]-pMin ) / ( pMax-pMin)
		

	def convertirImg_Txt (self, tailleCarac = 10):
		"""
		Convertir l'image en texte.
		
		Charge une image et la dcoupe en damier. Pour chaque case, la 
		valeur moyenne des pixels est calcule.
		
		@keyword tailleCarac: Taille des caractres en pixels.
		"""

		self.img = Image.open(self.nomImage)
		self.img = self.img.convert("RGB")
		
		# Image en 256 couleurs pour la sauvegarde en html et rtf (diminuer le nombre de couleurs)
		self.img256 = self.img.convert("P")
		
		tableCouleur = self.img256.resize((256, 1))
		tableCouleur.putdata(range(256))
		tableCouleur = tableCouleur.convert("RGB").getdata()
		
		self.dimImg = Vecteur2D(self.img.size[0], self.img.size[1])
		# Redimentionner l'image si sa largeur est plus petite que le nombre de caractres
		if self.dimImg.x < self.nbrCarac:
			ratio = self.nbrCarac / self.dimImg.x
			self.img = self.img.resize( ( int(self.dimImg.x*ratio), int(self.dimImg.y*ratio) ), Image.ANTIALIAS )
			self.dimImg = Vecteur2D(self.img.size[0], self.img.size[1])

		Ri = float(self.dimImg.x) / float(self.dimImg.y)
		# Ratio de la police
		# Multipli par 2. Evite les ratios trop levs pour les petites polices.
		font = ImageFont.truetype( self.parent.listeFonts[self.police], tailleCarac*2)
		tempo = Image.new('RGB', (20, 20), (255, 255, 255))
		car = ImageDraw.Draw(tempo)
		car.text((0, 0), 'S', font=font, fill=(0, 0, 0) )
		dim = car.textsize('s', font=font)
		ratioFont = float(dim[1]) / float(dim[0])
		
		i = self.dimImg.x/self.nbrCarac
		self.vNbrCarac = Vecteur2D( self.nbrCarac, self.nbrCarac / Ri )
		self.vNbrCarac.y = self.vNbrCarac.y / ratioFont
		j = self.dimImg.y / self.vNbrCarac.y
		self.dimDamier = Vecteur2D( i, j )

		# Variables pour rendre compte de la progression de la conversion (jauge de progression).
		progression = 0
		pasProgression = 100.0 / (self.vNbrCarac.x * self.vNbrCarac.y)

		self.vNbrCarac.x = int(self.vNbrCarac.x)
		self.vNbrCarac.y = int(self.vNbrCarac.y)
		# Poids noir/blanc de l'image pour chaque portion.
		self.tabImg = []
		# Couleur moyenne pour chaque portion
		self.tabCouleur = []
		# Idem mais en 256 couleurs (approximativement car moyenne des couleurs)
		self.tabCouleur256 = []
		# Parcourir l'image
		for ligne in range(self.vNbrCarac.y):
			self.tabImg.append( [] )
			self.tabCouleur.append( [] )
			self.tabCouleur256.append( [] )
			for colonne in range(self.vNbrCarac.x):
				# Mise  jour de la progression.
				self.parent.jauge.SetValue( int(progression) )
				progression += pasProgression
				# Lire la portion et calculer sa valeur
				a = int(colonne*self.dimDamier.x)
				b = int(ligne*self.dimDamier.y)
				c = int(colonne*self.dimDamier.x+self.dimDamier.x)
				d = int(ligne*self.dimDamier.y+self.dimDamier.y)
				portion = self.img.crop( (a, b, c, d) )
				por = portion.load()

				portion256 = self.img256.crop( (a, b, c, d) )
				por256 = portion256.load()

				poids = 0.00001
				poidsC = {'r': 0, 'v': 0, 'b': 0}
				poidsC256 = {'r': 0, 'v': 0, 'b': 0}
				# Parcourir chaque pixel de la portion.
				for y in range(portion.size[1]):
					for x in range(portion.size[0]):
						pix = por[x, y]
						pix256 = tableCouleur[por256[x, y]]
						p = ( pix[0]/255.0 + pix[1]/255.0 + pix[2]/255.0 ) / 3.0
						poids += p
						
						poidsC = {'r': poidsC['r']+pix[0], 'v': poidsC['v']+pix[1], 'b': poidsC['b']+pix[2] }
						poidsC256 = {'r': poidsC256['r']+pix256[0], 'v': poidsC256['v']+pix256[1], 'b': poidsC256['b']+pix256[2] }
				
				nbrPixelMax = ((c-a) * (d-b))
				poidsMoyen = float(poids) / nbrPixelMax
				self.tabImg[ligne].append( poidsMoyen )
				
				poidsMoyenC = {'r': poidsC['r'] / nbrPixelMax, 'v': poidsC['v'] / nbrPixelMax, 'b': poidsC['b'] / nbrPixelMax}
				self.tabCouleur[ligne].append( poidsMoyenC )

				poidsMoyenC256 = {'r': poidsC256['r'] / nbrPixelMax, 'v': poidsC256['v'] / nbrPixelMax, 'b': poidsC256['b'] / nbrPixelMax}
				self.tabCouleur256[ligne].append( poidsMoyenC256 )

		# Parcourir la table et remplacer par les caractres correspondants.
		self.tabAscii = []
		ligne = 0
		for y in self.tabImg:
			self.tabAscii.append( [] )
			for x in y:
				car = self.trouverCaractereProche ( self.poidsNBCarac, x, self.tolerancePoids )
				self.tabAscii[ligne].append( car )

			ligne += 1
		# Ecrire le texte
		txt = ''
		for y in self.tabAscii:
			for x in y:
				txt += x
			txt += '\n'

		self.parent.jauge.SetValue( 100 )

		return txt


	def convertirTxtRtf(self, fichier, texte):
		"""
		Convertir un texte en rtf.
		
		@param fichier: Nom et chemin de sauvegarde de l'image.
		@param texte: Texte  convertir en image.
		"""

		tailleTxt = self.parent.tailleCarac.GetValue() * 2
		# Crer un nouveau document
		doc = Document()
		ss = doc.StyleSheet
		# Ajouter les fonts (Lucida et Courier sont dja reconnues)
		f = Font( 'monofur', 'monofur' )
		ss.Fonts.append( f )
		f = Font( 'Bitstream Vera Sans Mono', 'Bitstream Vera Sans Mono' )
		ss.Fonts.append( f )
		f = Font( 'White Rabbit', 'White Rabbit' )
		ss.Fonts.append( f )
		f = Font( 'Crystal', 'Crystal' )
		ss.Fonts.append( f )
		f = Font( 'ModeNine', 'ModeNine' )
		ss.Fonts.append( f )

		self.fontsRTF = {'Monofur': ss.Fonts.monofur, 'Bitstream Vera Sans Mono': ss.Fonts.BitstreamVeraSansMono, 'White Rabbit': ss.Fonts.WhiteRabbit, 
		'Courier New': ss.Fonts.CourierNew, 'Crystal': ss.Fonts.Crystal, 'Lucida Console': ss.Fonts.LucidaConsole, 'ModeNine': ss.Fonts.ModeNine }

		section = Section()
		p = Paragraph()

		if self.parent.teinteCarac.GetSelection() == 0:

			lignes = texte.split("\n")
			for ligne in lignes:

				p.append( TEXT( str(ligne.encode('latin-1') ), LINE, font=self.fontsRTF[self.police], size=tailleTxt ) )

		else:
			if self.parent.teinteCarac.GetSelection() == 1:
				couleur = False
			else:
				couleur = True

			nouvelleTable = []
			# Index du nombre minimum de couleurs
			self.indexMin = []
		
			ii, jj = 0, 0
			for j in self.tabCouleur256:
				nouvelleTable.append( [] )
				for i in j:
					# Crer une couleur  partir de la couleur moyenne de la portion
					cCourante = Couleur(i['r'], i['v'], i['b'])
					# Vrai si la couleur est dja indexe
					existante = False
					index = 0
					for c in self.indexMin:
						# Test si la couleur existe. Large tolrance pour diminuer le nombre de couleurs diffrentes
						if c == cCourante:
							existante = True
							break
						index += 1
					# Si la couleur n'est pas encore indexe, on l'ajoute.
					if not existante:
						self.indexMin.append(cCourante)
					# Attribuer  la nouvelle table l'index de couleur pour le caractre courant
					nouvelleTable[jj].append( index )

					ii += 1
				jj += 1
		
			# Applatir le tableau de 256 couleurs (2D->1D)
			tabCRTF = []
			for y in nouvelleTable:
				for x in y:
					tabCRTF.append(x)
				tabCRTF.append(None)
			# Crer les modles de couleur pour le fichier rtf
			listeCol = []
			i = 0
			for s in range(len(self.indexMin)):
				if couleur:
					coul = Colour( str(s), self.indexMin[s].r, self.indexMin[s].v, self.indexMin[s].b)
				else:
					cg = int( (self.indexMin[s].r + self.indexMin[s].v + self.indexMin[s].b) / 3.0 )
					coul = Colour( str(s), cg, cg, cg)
				ss.Colours.append(coul)
				listeCol.append(coul)
				i += 1
			# Attribuer les couleurs, selon leur index,  chaque caractre
			i = 0
			for c in texte:
				if c != '\n':
					p.append( TEXT( str(c.encode('latin-1') ), font=self.fontsRTF[self.police], size=tailleTxt, colour=listeCol[ tabCRTF[i] ] ) )
				else:
					p.append( LINE )
				i += 1

		section.append(p)
		doc.Sections.append( section )
		
		# Sauvegarder le fichier rtf
		DR = Renderer()
		DR.Write( doc, OpenFile( fichier[:-4] ) )

	def convertirTxtImg(self, fichier, texte, format):
		"""
		Convertir un texte en image.
		
		@param fichier: Nom et chemin de sauvegarde de l'image.
		@param texte: Texte  convertir en image.
		@param format: Format de sauvegarde ('PNG', 'JPEG', etc.)
		"""
		# La taille des caractres est multiplie par 4 pour un meilleur rendu (une image finale plus grande)
		taillePolice = self.parent.tailleCarac.GetValue() * 2 # * 4
		texte = self.convertirImg_Txt(tailleCarac=taillePolice)

		# Rcuprer chaque ligne du texte.
		lignes = texte.split('\n')
		# Dessiner la premire ligne
		font = ImageFont.truetype( self.parent.listeFonts[self.police], taillePolice)
		tempo = Image.new('RGB', (20, 20), (255, 255, 255))
		car = ImageDraw.Draw(tempo)
		car.text((0, 0), lignes[0], font=font, fill=(0, 0, 0) )
		# Rcuprer la taille de la premire ligne (aussi longue que les suivantes).
		dim = car.textsize(lignes[0], font=font)
		# Dimensions de l'image finale.
		dimX = dim[0]
		dimY = dim[1] * len(lignes)
		# Crer une image pour sauver le rsultat.
		resultat = Image.new('RGB', (dimX, dimY), (255, 255, 255))
		
		if self.parent.teinteCarac.GetSelection() == 0:
			# Noir / blanc
			ligneX = dim[0]
			ligneY = dim[1]
			posY = 0
			for ligne in lignes:
				# Dessiner la ligne courante.
				img = Image.new('RGB', (ligneX, ligneY), (255, 255, 255)) 
				carac = ImageDraw.Draw(img)
				carac.text((0, 0), ligne, font=font, fill=(0, 0, 0) )
				# Copier la ligne sur l'image de rsultat.
				resultat.paste( img, (0, posY) )
				# Passer  la ligne suivante.
				posY += ligneY
		else:
			
			if self.parent.teinteCarac.GetSelection() == 1:
				couleur = False
			else:
				couleur = True
			
			tabC = []
			for y in self.tabCouleur:
				for x in y:
					tabC.append(x)
			
			font = ImageFont.truetype( self.parent.listeFonts[self.police], taillePolice)
			tempo = Image.new('RGB', (20, 20), (255, 255, 255))
			car = ImageDraw.Draw(tempo)
			car.text((0, 0), lignes[0][0], font=font, fill=(0, 0, 0) )
			# Rcuprer la taille de la premire ligne (aussi longue que les suivantes).
			dimCar = car.textsize(lignes[0][0], font=font)

			# Couleur ou niveau de gris
			ligneX = dim[0]
			ligneY = dim[1]
			posX = 0
			posY = 0
			i = 0
			for ligne in lignes:
				img = Image.new('RGB', (dimCar[0], dimCar[1]), (255, 255, 255)) 
				# Dessiner le caractre courant.
				for c in ligne:

					img = Image.new('RGB', (dimCar[0], dimCar[1]), (255, 255, 255))
					carac = ImageDraw.Draw(img)
					if couleur:
						carac.text((0, 0), c, font=font, fill=(tabC[i]['r'], tabC[i]['v'], tabC[i]['b']) )
					else:
						cg = int( (tabC[i]['r'] + tabC[i]['v'] + tabC[i]['b']) / 3.0 )
						carac.text((0, 0), c, font=font, fill=(cg, cg, cg) )
					# Copier la ligne sur l'image de rsultat.
					resultat.paste( img, (posX, posY) )
					posX += dimCar[0]

					i += 1

				posY += dimCar[1]
				posX = 0


		# Sauver l'image
		resultat.save( fichier, format)
		
	def trouverCaractereProche ( self, table, poids, tolerance ):
		"""
		Cherche le caractre le plus proche d'un poids donn.
		
		@param table: Dictionnaire des caractres et de leurs poids respectifs {'a': 0.456, 'b': 0.563, ... }
		@param poids: Poids dont on cherche la correspondance avec la table.
		@param tolerance: Diffrence de poids acceptable entre le poids cherch et le poids test.
		@return: Le caractre le plus proche du poids entr en paramtre.
		"""
		for t in table:
			if poids <= (table[t]+tolerance) and poids >= (table[t]-tolerance):
				return t
		# Si on ne trouve pas de correspondance, on retourne le caractre par dfaut.
		return self.caractereDefaut