# -*- coding:latin-1 -*-
"""
Copyright 2009 Sbastien L. W. Baelde (contact : clic4@clic4.org)

    This file is part of ImASCII.

    ImASCII is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    ImASCII is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ImASCII.  If not, see <http://www.gnu.org/licenses/>.
"""

from ImageASCII import ImageASCII
from wx import wx
from PIL import Image
from wx.lib.wordwrap import wordwrap
import os, sys
import math

txtLicence = """    ImASCII is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    ImASCII is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ImASCII.  If not, see <http://www.gnu.org/licenses/>.

		"""

class Fenetre(wx.Frame):
	"""
	Convertisseur d'image en ASCII.

	ImageASCII permet de convertir une image en caractres. Le rsultat est sauv soit sous la forme 
	d'un texte (.txt, .rtf), soit d'une image (.jpg, .png, .gif).
	
	Bugs/To do:
	
		* [OK] Adaptation des dimensions de l'image en fonction du nbr de caractres (images trop petite)
		* [OK] Jeu de caractres personnaliss
		* [OK] Corriger l'erreur de sauvegarde en image pour certains caractres
		* [OK] Sauvegarde rtf
		* [OK] [OPTIMISABLE] Adapter la taille du rsultat pour tenir compte du ratio de la police
		* [OK] Adapter la plage de prcision
		* [OK] Rduire le nombre de couleurs pour la sauvegarde rtf (chargement lent car trop de styles de couleur)
		* Etoffer le menu et touches raccourcis pour le menu (F1, etc...)
		* [OK] Mj de l'aide
		* [OK] Ajouter le mode de conversion par valeur de niveau de gris et couleur
		* [OK] Bugs d'affichages : passage aide/resultat (dim de la zone) ; mj de la barre de status
		* [OK] Fentre minimum trop grande ! (portable: 1100x780 ..)
		* Drag and Drop ne fonctione pas sous linux/wine
		* [OK] Avec wine, les fonts courrier et lucida n'existent pas : viter le plantage
	
	"""
	def __init__(self, parent):
		"""
		Initialisation de l'interface.
		
		Cration de controleurs pour ajuster les paramtres de la conversion.
		
		@param parent: Fentre parent.
		"""
		self.version = '0.70'
		wx.Frame.__init__(self, parent, -1, 'ImASCII ' + self.version, (0, 0), (950, 750), wx.SYSTEM_MENU | wx.CAPTION | wx.CLOSE_BOX | wx.CLIP_CHILDREN | wx.RESIZE_BORDER | wx.MINIMIZE_BOX | wx.MAXIMIZE_BOX, 'fen')
		self.SetMinSize((950, 750))
		self.CentreOnScreen()

		# Dossier
		try:
			self.cheminProg = os.path.dirname(sys.argv[0])
			os.chdir(self.cheminProg)
		except:
			self.cheminProg = "./"

		# Liste de fonts et noms de fichiers associs
		if os.name  == 'nt':
			self.listeFonts = {'Courier New': 'cour.ttf', 'Lucida Console': 'lucon.ttf', 'Bitstream Vera Sans Mono': 'VeraMono.ttf', 'Crystal': 'crysrg__.ttf', 
			'ModeNine': 'modenine.ttf', 'Monofur': 'monof55.ttf', 'White Rabbit': 'whitrabt.ttf'}
		else:
			self.listeFonts = {'Bitstream Vera Sans Mono': 'VeraMono.ttf', 'Crystal': 'crysrg__.ttf', 
			'ModeNine': 'modenine.ttf', 'Monofur': 'monof55.ttf', 'White Rabbit': 'whitrabt.ttf'}

		self.nomFichierSauve = 'resultat'

		# Icone de fentre
		icone = wx.Icon( 'img/imascii.png', wx.BITMAP_TYPE_PNG )
		self.SetIcon(icone)

		self.fenetre = wx.Panel(self, size=(-1, -1))		# Panneau global de la fentre
		
		self.imgAscii = ImageASCII(self)
		cadreCharger = wx.StaticBox(self.fenetre, -1, u"Image originale")
		cadreParam = wx.StaticBox(self.fenetre, -1, u"Paramtres")
		
		# Aperu de l'image charge
		imgPil = Image.open('img/guttenberg.png')
		imgPil.thumbnail((200, 100), Image.ANTIALIAS)
		imgWx = wx.EmptyImage(imgPil.size[0], imgPil.size[1])
		imgWx.SetData(imgPil.convert("RGB").tostring())
		imgWx.SetAlphaData(imgPil.convert("RGBA").tostring()[3::4])
		img = wx.BitmapFromImage(imgWx)
		
		self.apercuImg = wx.StaticBitmap(self.fenetre, -1, img, size=(img.GetWidth(), img.GetHeight()))
		# Bouton de chargement d'une image.
		bmp = wx.Bitmap("img/chargerImage.png", wx.BITMAP_TYPE_PNG)
		self.bChargerImage = wx.BitmapButton(self.fenetre, -1, bmp, (20, 20), (bmp.GetWidth()+10, bmp.GetHeight()+10))
		self.bChargerImage.SetToolTipString("Charger une image.")
		self.Bind(wx.EVT_BUTTON, self.eChargerImage, self.bChargerImage)

		# Boutons
		self.bConvertir = wx.Button(self.fenetre, -1, "Convertir en texte")
		self.bSauver = wx.Button(self.fenetre, -1, "Enregistrer")
		self.txtCheminImg = wx.TextCtrl(self.fenetre, -1, u"...", size=(200, 20))
		self.Bind(wx.EVT_BUTTON, self.eConvertirImg_Txt, self.bConvertir)
		self.Bind(wx.EVT_BUTTON, self.eSauverResultat, self.bSauver)

		sizerChargerImage = wx.BoxSizer(wx.HORIZONTAL)
		sizerChargerImage.Add(self.bChargerImage, 0, wx.SHAPED)
		sizerChargerImage.AddSpacer(5)
		sizerChargerImage.Add(self.txtCheminImg, 0, wx.SHAPED)

		self.sizerImage = wx.StaticBoxSizer(cadreCharger, wx.VERTICAL)
		self.sizerImage.Add(self.apercuImg, 0, wx.EXPAND | wx.ALIGN_CENTER_HORIZONTAL)
		self.sizerImage.AddSpacer(10)
		self.sizerImage.Add(sizerChargerImage, 0, wx.SHAPED)
	
		
		f = open('guttenberg.txt')
		txtIntro = f.read()
		f.close()
		# Fentre de texte pour le rendu
		self.ascii = wx.TextCtrl(self.fenetre, -1, txtIntro, size=(700, 500), style=wx.TE_MULTILINE|wx.TE_PROCESS_ENTER|wx.TE_RICH | wx.HSCROLL )
		self.aide = wx.TextCtrl(self.fenetre, -1, u"AIDE...", size=(-1, -1), style=wx.TE_MULTILINE|wx.TE_PROCESS_ENTER|wx.TE_RICH)
		self.aide.Show(False)

		self.glisserDeposer = GlisserFichiers(self)
		self.fenetre.SetDropTarget(self.glisserDeposer)
		# Glissire pour rgler la tolrance de conversion
		self.sliderTolerance = wx.Slider(self.fenetre, -1, 5, 1, 100, size=(220, -1), style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.Bind (wx.EVT_SCROLL_CHANGED, self.eChangerTolerance, self.sliderTolerance)
		# Glissire pour rgler le nombre de caractres horizontaux
		self.sliderNbrCarac = wx.Slider(self.fenetre, -1, 120, 30, 200, size=(220, -1), style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.Bind (wx.EVT_SCROLL_CHANGED, self.eChangerNbrCarac, self.sliderNbrCarac)
		
		txtTolerance = wx.StaticText(self.fenetre, -1, "Tolrance" )
		txtNbrCarac = wx.StaticText(self.fenetre, -1, "Nbr de caractres", size=(150, -1) )

		sizerCommandes = wx.BoxSizer(wx.VERTICAL)
		sizerCommandes.Add(txtNbrCarac, 0, wx.SHAPED)
		sizerCommandes.Add(self.sliderNbrCarac, 0, wx.SHAPED)
		sizerCommandes.Add(txtTolerance, 0, wx.SHAPED)
		sizerCommandes.Add(self.sliderTolerance, 0, wx.SHAPED)
		
		# Action sur les caractres : liste de caractres, caractre de fond, taille
		txtCaracFond = wx.StaticText(self.fenetre, -1, "Caractre par dfaut ")
		txtListeCarac = wx.StaticText(self.fenetre, -1, "Liste de caractres ")
		txtTailleCarac = wx.StaticText(self.fenetre, -1, "Taille des caractres " )
		
		self.caracFond = wx.TextCtrl(self.fenetre, -1, self.imgAscii.caractereDefaut, size=(20, -1))
		self.Bind (wx.EVT_TEXT, self.eChangerCaracDefaut, self.caracFond)
		
		self.typeListeCarac = wx.Choice(self.fenetre, -1, (100, 50), choices = ['Complte', 'Restreinte', "Personalise"])
		self.typeListeCarac.SetSelection(0)
		self.Bind (wx.EVT_CHOICE, self.eChangerListeCarac, self.typeListeCarac)
		
		self.tailleCarac = wx.SpinCtrl(self.fenetre, -1, size=(50, -1))
		self.Bind (wx.EVT_SPINCTRL, self.eChangerTailleCarac, self.tailleCarac)
		self.tailleCarac.SetRange(1, 24)
		self.tailleCarac.SetValue(5)
		
		self.teinteCarac = wx.RadioBox(self.fenetre, -1, "Teinte des caractres", (-1, -1), (-1, -1), ['Noir / blanc', 'Niveaux de gris', 'Couleur'], style=wx.RA_SPECIFY_ROWS)
		self.teinteCarac.SetSelection(0)
		
		# Affichage des polices/fonts disponibles
		listeF = self.listeFonts.keys()
		listeF.sort()
		self.nomFont = wx.ListBox(self.fenetre, -1, (-1, -1), (-1, 100), listeF, wx.LB_SINGLE)
		self.nomFont.SetSelection(3)
		self.Bind(wx.EVT_LISTBOX, self.eChangerFont, id=self.nomFont.GetId())
		
		sizerOptions = wx.FlexGridSizer(3, 2, vgap=5, hgap=20)
		sizerOptions.Add(txtListeCarac, 0, wx.SHAPED)
		sizerOptions.Add(self.typeListeCarac, 0, wx.SHAPED | wx.ALIGN_LEFT)
		sizerOptions.Add(txtTailleCarac, 0, wx.SHAPED)
		sizerOptions.Add(self.tailleCarac, 0, wx.SHAPED | wx.ALIGN_LEFT)
		sizerOptions.Add(txtCaracFond, 0, wx.SHAPED)
		sizerOptions.Add(self.caracFond, 0, wx.SHAPED | wx.ALIGN_LEFT)

		sizerCaractereHaut = wx.BoxSizer(wx.HORIZONTAL)
		sizerCaractereHaut.Add(sizerOptions, 0, wx.SHAPED)
		
		sizerCaractere = wx.StaticBoxSizer(cadreParam, wx.VERTICAL)
		sizerCaractere.Add(sizerCaractereHaut, 0, wx.SHAPED | wx.ALIGN_CENTER_HORIZONTAL)
		sizerCaractere.AddSpacer(15)
		sizerCaractere.Add(self.nomFont, 0, wx.SHAPED | wx.ALIGN_CENTER_HORIZONTAL )
		sizerCaractere.AddSpacer(10)
		sizerCaractere.Add(self.teinteCarac, 0, wx.SHAPED| wx.ALIGN_CENTER_HORIZONTAL)
		sizerCaractere.AddSpacer(15)
		sizerCaractere.Add(sizerCommandes, 0, wx.SHAPED| wx.ALIGN_CENTER_HORIZONTAL)

		sizerGlobal = wx.BoxSizer(wx.VERTICAL)
		sizerGlobal.AddSpacer(10)
		sizerGlobal.Add(self.sizerImage, 0, wx.EXPAND)
		sizerGlobal.AddSpacer(10)
		sizerGlobal.Add(sizerCaractere, 0, wx.EXPAND)
		sizerGlobal.AddSpacer(10)
		sizerGlobal.Add(self.bConvertir, 0, wx.SHAPED | wx.ALIGN_CENTER_HORIZONTAL)
		sizerGlobal.AddSpacer(15)
		sizerGlobal.Add(self.bSauver, 0, wx.SHAPED | wx.ALIGN_CENTER_HORIZONTAL)
		
		sizerAction = wx.BoxSizer(wx.HORIZONTAL)
		sizerAction.AddSpacer(20)
		sizerAction.Add(sizerGlobal, 0, wx.EXPAND)
		sizerAction.AddSpacer(20)
		
		sizerFenetre = wx.BoxSizer(wx.HORIZONTAL)
		sizerFenetre.Add(self.ascii, 1, wx.EXPAND)
		sizerFenetre.Add(self.aide, 1, wx.EXPAND)
		sizerFenetre.AddSpacer(20)
		sizerFenetre.Add(sizerAction, 0, wx.EXPAND)
		
		self.fenetre.SetSizer(sizerFenetre)
		
		# Barre Statut
		self.sb = wx.StatusBar(self, -1 )
		self.sb.SetFieldsCount(2)
		self.sb.SetStatusWidths([150, -1])
		self.sb.SetStatusText(u" Prt", 0)

		# Jauge
		self.jauge = wx.Gauge(self.sb, -1, 100, (160, 5), size=(650, 12))
		self.jauge.SetValue(0)
		self.SetStatusBar(self.sb)

		# Menu
		menuBar = wx.MenuBar()
		# Menu fichier
		menu1 = wx.Menu()
		menu1.Append(99, "Ouvrir...\tCtrl+O", " Ouvrir une image")
		menu1.Append(100, "Enregistrer sous...\tCtrl+S", " Enregistrer le rsultat")
		menu1.Append(103, "Quitter\tCtrl+Q", " Quitter le logiciel")
		menuBar.Append(menu1, "Fichier")
		# Menu d'aide
		menu2 = wx.Menu()
		menu2.Append(201, "Contenu de l'aide\tF1", " Aide pour utiliser le logiciel")
		menu2.Append(202, "A propos de ImASCII", " Informations sur le logiciel")
		menuBar.Append(menu2, "Aide")

		self.SetMenuBar(menuBar)

		self.Bind(wx.EVT_MENU, self.eChargerImage, id=99)
		self.Bind(wx.EVT_MENU, self.eSauverResultat, id=100)
		self.Bind(wx.EVT_MENU, self.eFermerProg, id=103)
		self.Bind(wx.EVT_MENU, self.eAide, id=201)
		self.Bind(wx.EVT_MENU, self.OnAbout, id=202)
		self.Bind(wx.EVT_MENU_CLOSE, self.eMajSB)

		font = wx.Font(4, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL , False, faceName=self.imgAscii.police)
		self.ascii.SetFont(font)
		
		self.SetFocus()

	def eMajSB (self, event):
		""" Mise  jour de la barre de status. """
		self.sb.SetStatusText(u" Prt", 0)

	def eChangerFont (self, event):
		"""
		Changer la police de caractre utilise.
		
		Liste de fonts disponibles: Courier New, Lucida Console, Bitstream Vera Sans Mono, Crystal, ModeNine, monofur, White Rabbit.
		"""
		face = self.nomFont.GetStringSelection()
		self.imgAscii.police = face

	def eChangerListeCarac (self, event):
		"""
		Changer la liste des caractres employs lors de la conversion.
		"""
		choix = self.typeListeCarac.GetSelection()
		if choix == 0:
			self.imgAscii.listeCaracteres = self.imgAscii.listeCaracteresA
		elif choix == 1:
			self.imgAscii.listeCaracteres = self.imgAscii.listeCaracteresB
		elif choix == 2:
			
			# Caractres personaliss
			carac = ''
			for c in self.imgAscii.listeCaracteresPerso:
				carac += c + ', '
			carac = carac[:-2]
			
			dlg = CaracPerso(self)
			dlg.CenterOnScreen()
			dlg.carac.SetValue(carac)
			res = dlg.ShowModal()
			car = dlg.carac.GetValue()
			liste = car.split(',')
			self.imgAscii.listeCaracteresPerso = []
			for c in liste:
				if c == '  ':
					self.imgAscii.listeCaracteresPerso.append(' ')
				else:
					cOk = c.replace(' ', '')
					cOk = cOk.replace('\n', '')
					self.imgAscii.listeCaracteresPerso.append( cOk )
			self.imgAscii.listeCaracteres = self.imgAscii.listeCaracteresPerso
			dlg.Destroy()
			
		self.imgAscii.calculerPoids()

	def eChangerTailleCarac (self, event):
		"""	
		Changer la taille du texte d'affichage.
		"""

		pass

	def eChangerCaracDefaut (self, event):
		"""
		Changer le caractre par dfaut.
		"""
		car = self.caracFond.GetValue().decode('utf-8').encode('cp1252')
		if car == '':
			self.imgAscii.caractereDefaut = ' '
		else:
			self.imgAscii.caractereDefaut = car

	def eChangerTolerance (self, event):
		"""
		Changer la tolrance sur le poids.
		"""
		self.imgAscii.tolerancePoids = float(self.sliderTolerance.GetValue()) / (self.imgAscii.precisionTolerance*1.5)

	def eChangerNbrCarac (self, event):
		"""
		Changer le nombre de caractres utiliss pour reprsenter l'image.
		"""
		self.imgAscii.nbrCarac = float(self.sliderNbrCarac.GetValue())

	def eConvertirImg_Txt (self, event):
		"""
		Convertit l'image charge en mmoire en texte.
		"""
		if os.path.exists(self.imgAscii.nomImage):
			self.sb.SetStatusText(u" Conversion en cours...", 0)
			res = self.imgAscii.convertirImg_Txt(tailleCarac=self.tailleCarac.GetValue())
			listeChoix = ['nb', 'ng', 'co']
			self.afficherTexte(res, couleur=listeChoix[self.teinteCarac.GetSelection()])

	def eSauverResultat (self, event):
		"""
		Enregistre le rsultat de la conversion dans le format appropri.
		"""
		filtre = "Texte brute (*.txt)|*.txt|"	\
		"Rich Text Format (*.rtf)|*.rtf|"	\
		"Image JPG (*.jpg)|*.jpg|"    		\
		"Image PNG (*.png)|*.png|"      	\
		"Image GIF (*.gif)|*.gif|"		\
		"Image BMP (*.bmp)|*.bmp"

		dlg = wx.FileDialog(self.fenetre, message="Enregistrer sous ...", defaultDir=self.cheminProg+"/art ASCII", defaultFile=self.nomFichierSauve, wildcard=filtre, style=wx.SAVE)

		#dlg.SetFilterIndex(1)

		if dlg.ShowModal() == wx.ID_OK:
			path = dlg.GetPath()

		dlg.Destroy()
		
		try:
			self.sb.SetStatusText(u" Sauvegarde...", 0)
			choix = path[len(path)-3:]
			if choix == 'txt':
				# TXT
				fichier = open( path, 'w')
				for y in self.imgAscii.tabAscii:
					for x in y:
						fichier.write(x)
					fichier.write('\n')
				fichier.close()

			elif choix == 'rtf':
				# RTF
				self.imgAscii.convertirTxtRtf( path, self.ascii.GetValue() )
			else:
				if choix == 'jpg':
					choix = 'jpeg'
				self.imgAscii.convertirTxtImg( path, self.ascii.GetValue(), choix.upper() )

		except:
			pass

		self.sb.SetStatusText(u" Prt", 0)

	def eChangerDossierSauvegarde(self, event):
		"""
		Changer le dossier de sauvegarde.
		"""
		dlg = wx.DirDialog(self, message="Choisissez le rpertoire de sauvegarde" )
		if dlg.ShowModal() == wx.ID_OK:
			# Rcuprer le dossier slectionn.
			chemin = dlg.GetPath()
			self.txtCheminSauver.SetValue(chemin + "\\")

		dlg.Destroy()

	def eChargerImage(self, event):
		"""
		Dfinir le chemin de l'image charg.
		"""
		dlg = wx.FileDialog(self, message="Choisissez un fichier image", defaultDir=os.getcwd(), defaultFile="", style=wx.OPEN )
		if dlg.ShowModal() == wx.ID_OK:
			paths = dlg.GetPaths()
			chemin = paths[0]
			self.txtCheminImg.SetValue(chemin)
			self.imgAscii.nomImage = chemin
			nom = os.path.basename(chemin)
			nom = nom.split('.')
			self.nomFichierSauve = nom[0]+"_img"
			
			self.majApercuImage(self.imgAscii.nomImage)
			
		dlg.Destroy()

	def majApercuImage (self, chemin):
		"""
		Mise  jour de l'aperu de l'image charge. 
		
		@param chemin: Nom et chemin de l'image charge.
		"""
		imgPil = Image.open(chemin)
		imgPil.thumbnail((200, 100), Image.ANTIALIAS)
		imgWx = wx.EmptyImage(imgPil.size[0], imgPil.size[1])
		imgWx.SetData(imgPil.convert("RGB").tostring())
		imgWx.SetAlphaData(imgPil.convert("RGBA").tostring()[3::4])
		img = wx.BitmapFromImage(imgWx)
			
		self.apercuImg.SetBitmap(img)
		# Donner la nouvelle taille minimum pour centrage correct
		self.sizerImage.SetItemMinSize( self.apercuImg, self.apercuImg.GetSize() )
		self.sizerImage.Layout()

	def OnAbout(self, event):
		"""
		Message d'informations sur le logiciel.
		"""
		self.sb.SetStatusText(u" Informations sur le logiciel", 0)
		info = wx.AboutDialogInfo()
		info.Name = u"ImASCII"
		info.Version = str(self.version)
		info.Copyright = "Copyright (C) 2009 Sbastien L. W. Baelde"
		info.Description = wordwrap("Convertisseur d'image en texte.", 500, wx.ClientDC(self))
		info.WebSite = ("http://clic4.org/imascii", "Site web: http://clic4.org/imascii")
		info.Developers = [ "Sbastien L. W. Baelde (clic4@clic4.org)" ]
		info.License = wordwrap(txtLicence, 800, wx.ClientDC(self))

		wx.AboutBox(info)
		self.sb.SetStatusText(u" Prt", 0)

	def eAide (self, event):
		"""
		Documentation sur le logiciel.
		"""
		os.startfile('documentation.pdf')
		
	def eFermerProg(self, event):
		self.Close()

	def afficherTexte (self, texte, couleur='nb'):
		"""
		Afficher le rsultat dans la zone de texte.
		
		@param texte: Le texte  afficher.
		@keyword couleur: nb=noir/blanc, ng=niveau de gris, co=couleur
		"""
		
		self.ascii.Show(True)
		self.aide.Show(False)

		self.ascii.Clear()
		texte = texte.decode('cp1252')
		font = wx.Font(self.tailleCarac.GetValue(), wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, False, faceName=self.imgAscii.police)
		self.ascii.SetFont(font)
		self.ascii.SetValue(texte)

		tabC = []
		for y in self.imgAscii.tabCouleur:
			for x in y:
				tabC.append(x)
			tabC.append(None)

		if couleur == 'co':

			self.sb.SetStatusText(u" Colorisation...", 0)
			self.jauge.SetValue( 0 )
			pas = 100.0 / len(tabC)
			progression = 0.0
			i = 0
			txt = wx.TextAttr()
			for c in texte:

				if c != '\n':
					couleur = wx.Colour(int(tabC[i]['r']), int(tabC[i]['v']), tabC[i]['b'] )
					txt.SetTextColour(couleur)
					self.ascii.SetStyle( i, i+1, txt)
					self.jauge.SetValue( int(progression) )
					progression += pas
				i += 1

		elif couleur == 'ng':

			self.sb.SetStatusText(u" Colorisation...", 0)
			self.jauge.SetValue( 0 )
			pas = 100.0 / len(tabC)
			progression = 0.0
			i = 0
			txt = wx.TextAttr()
			for c in texte:
				if c != '\n':
					cg = int( (tabC[i]['r'] + tabC[i]['v'] + tabC[i]['b']) / 3.0 )
					couleur = wx.Colour(cg, cg, cg )
					txt.SetTextColour(couleur)
					self.ascii.SetStyle( i, i+1, txt)
					self.jauge.SetValue( int(progression) )
					progression += pas
				i += 1

		elif couleur == 'nb':
			# Noir / blanc
			pass

		self.ascii.SetSelection(-1, -1)
		self.ascii.SetSelection(0, 0)

		self.jauge.SetValue( 100 )
		self.sb.SetStatusText(u" Prt", 0)

class GlisserFichiers(wx.FileDropTarget):
	"""
	Classe personalise pour le glisser/dposer de fichiers.
	"""
	def __init__(self, parent):
		"""
		Constructeur.
		
		@param parent: Fentre parent de l'objet.
		"""
		wx.FileDropTarget.__init__(self)
		self.parent = parent

	def OnDropFiles(self, x, y, chemin):
		"""
		Dposer un fichier.
		
		@param x: Position x.
		@param y: Position y.
		@param chemin: Chemin et nom du fichier.
		"""

		self.parent.txtCheminImg.SetValue(chemin[0] )
		self.parent.imgAscii.nomImage = chemin[0]
		nom = os.path.basename(chemin[0])
		nom = nom.split('.')
		self.parent.nomFichierSauve = nom[0]+"_img"
		self.parent.majApercuImage(self.parent.imgAscii.nomImage)


class CaracPerso (wx.Dialog):
	""" Fentre popup permettant d'entrer une liste de caractres. """
	def __init__ (self, parent):
		
		wx.Dialog.__init__(self, parent, -1, u"Liste personalise de caractres", pos = wx.DefaultPosition, size = (250, 200), style = wx.DEFAULT_DIALOG_STYLE)

		self.fenetre = wx.Panel(self, size=(250, 200))
		self.carac = wx.TextCtrl(self.fenetre, -1, u"", size=(250, 150), style=wx.TE_MULTILINE)
		self.bOk = wx.Button(self.fenetre, wx.ID_OK)
		
		sizer = wx.BoxSizer(wx.VERTICAL)
		sizer.Add(self.carac, 0, wx.EXPAND)
		sizer.Add(self.bOk, 0, wx.EXPAND)
		
		self.SetSizer(sizer)

class ImgTxt(wx.App):
	"""
	Classe de l'application.
	"""
	def OnInit(self):
		
		fen = Fenetre(None)
		self.SetTopWindow(fen)

		fen.Show(True)
		
		return True
		

def main():
	# Crer l'application et lancer la boucle principale.
	app = ImgTxt(redirect=False)
	app.MainLoop()


if __name__ == '__main__': main()